/**
 *    @file  rptcluster.c
 *   @brief
 *
 *  <+DETAILED+>
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  29/10/2024
 *      Revision:  none
 * Last modified:  2024-11-06 19:21
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "rptcluster.h"
#include "clustercon.h"
#include "json.h"
#include "csv.h"
#include <string.h>

/* To refactor in a report common file */
static void csv_addjsonfield(FILE* reportfile, const cJSON* json, char* fieldname) {
    char* text = json2text(cJSON_GetObjectItemCaseSensitive(json, fieldname));
    csv_addfield(reportfile,text);
    free(text);
}

void report_cluster_header(FILE* reportfile) {
    fprintf(reportfile,
            "cluster_host,name,rack_aware,created_time,free_memory,"
            "available_memory,available_memory_no_overbooking,available_flash,"
            "available_flash_no_overbooking,ephemeral_storage_avail,"
            "ephemeral_storage_free,persistent_storage_avail,"
            "persistent_storage_free,provisional_flash,"
            "provisional_flash_no_overbooking,provisional_memory,"
            "provisional_memory_no_overbooking,activation_date,expiration_date,"
            "cluster_name,owner,features,shards_limit,ram_shards_in_use,"
            "ram_shards_limit,flash_shards_in_use,flash_shards_limit,expired\r\n"
           );
}

static cJSON* report_querygetjson(const cluster_t* cluster, const char* endpoint) {
    rsclustercon_t* rsclustercon;

    if (NULL==(rsclustercon = cluster_new(cluster))) {
        fprintf(stderr,"report_querygetjson cluster_new failed\n");
        return NULL;
    }
    if (0!=cluster_open(rsclustercon)) {
        fprintf(stderr,"report_querygetjson cluster_open failed\n");
        cluster_del(rsclustercon);
        return NULL;
    }

    cJSON* retval;
    if (NULL==(retval = cluster_queryget(rsclustercon, endpoint))) {
        const char *error_ptr = cJSON_GetErrorPtr();
        if (error_ptr != NULL)
            fprintf(stderr, "Error before: %s\n", error_ptr);
        retval = cJSON_CreateObject();
    }

    cluster_close(rsclustercon);
    cluster_del(rsclustercon);

    return retval;
}

void report_cluster(FILE* reportfile, const cluster_t* cluster) {
    cJSON* cluster_json;
    cJSON* clusterstats_json;
    cJSON* license_json;

    cluster_json = report_querygetjson(cluster, "/v1/cluster");
    clusterstats_json = report_querygetjson(cluster, "/v1/cluster/stats/last");
    license_json = report_querygetjson(cluster, "/v1/license");

    csv_addfield(reportfile,cluster->host);
    csv_addjsonfield(reportfile, cluster_json, "name");
    csv_addjsonfield(reportfile, cluster_json, "rack_aware");
    csv_addjsonfield(reportfile, cluster_json, "created_time");
    csv_addjsonfield(reportfile, clusterstats_json, "free_memory");
    csv_addjsonfield(reportfile, clusterstats_json, "available_memory");
    csv_addjsonfield(reportfile, clusterstats_json, "available_memory_no_overbooking");
    csv_addjsonfield(reportfile, clusterstats_json, "available_flash");
    csv_addjsonfield(reportfile, clusterstats_json, "available_flash_no_overbooking");
    csv_addjsonfield(reportfile, clusterstats_json, "ephemeral_storage_avail");
    csv_addjsonfield(reportfile, clusterstats_json, "ephemeral_storage_free");
    csv_addjsonfield(reportfile, clusterstats_json, "persistent_storage_avail");
    csv_addjsonfield(reportfile, clusterstats_json, "persistent_storage_free");
    csv_addjsonfield(reportfile, clusterstats_json, "provisional_flash");
    csv_addjsonfield(reportfile, clusterstats_json, "provisional_flash_no_overbooking");
    csv_addjsonfield(reportfile, clusterstats_json, "provisional_memory");
    csv_addjsonfield(reportfile, clusterstats_json, "provisional_memory_no_overbooking");
    csv_addjsonfield(reportfile, license_json, "activation_date");
    csv_addjsonfield(reportfile, license_json, "expiration_date");
    csv_addjsonfield(reportfile, license_json, "cluster_name");
    csv_addjsonfield(reportfile, license_json, "owner");
    csv_addjsonfield(reportfile, license_json, "features");
    csv_addjsonfield(reportfile, license_json, "shards_limit");
    csv_addjsonfield(reportfile, license_json, "ram_shards_in_use");
    csv_addjsonfield(reportfile, license_json, "ram_shards_limit");
    csv_addjsonfield(reportfile, license_json, "flash_shards_in_use");
    csv_addjsonfield(reportfile, license_json, "flash_shards_limit");
    csv_addjsonfield(reportfile, license_json, "expired");
    csv_addline(reportfile);

    cJSON_Delete(cluster_json);
    cJSON_Delete(clusterstats_json);
    cJSON_Delete(license_json);
}


/*



clusterdef.host
nodes.uid
nodes.addr
nodes.external_addr
nodes.cores
nodes.total_memory
nodes.ephemeral_storage_size
nodes.persistent_storage_size
nodes.os_version
nodes.rack_id
nodes.max_listeners
nodes.max_redis_servers
nodes.ram_shard_count
nodes.flash_shard_count
nodes.shard_count
nodes.software_version
nodes.uptime
nodes.accept_servers
nodes.status
nodesstatus.cores
nodesstatus.free_provisional_ram
nodesstatus.free_ram
nodesstatus.hostname
nodesstatus.node_overbooking_depth
nodesstatus.node_status
nodesstatus.role
nodesstatus.software_version
nodesstatus.total_memory
nodesstatus.total_provisional_ram
nodesstatslast.available_memory
nodesstatslast.available_memory_no_overbooking
nodesstatslast.ephemeral_storage_avail
nodesstatslast.ephemeral_storage_free
nodesstatslast.free_memory
nodesstatslast.persistent_storage_avail
nodesstatslast.persistent_storage_free
nodesstatslast.provisional_memory
nodesstatslast.provisional_memory_no_overbooking
*/
/* vim: set tw=80: */
