/**
 *    @file  sclist.c
 *   @brief  Basic single chained generic list
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  15/11/2024
 *      Revision:  none
 * Last modified:  2024-11-29 17:58
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "sclist.h"

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

/** Private list record structure
 */
typedef struct sclistrecord_s {
    void* value;                                /**< Pointer to value */
    struct sclistrecord_s* next;                 /**< Next record in the list */
} sclistrecord_t;

/** Opaque sclist structure
 */
typedef struct sclist_s {
    sclistrecord_t* first;                       /**< Pointer to the first record */
    sclistrecord_t* last;                        /**< Pointer to the last record */
} sclist_t;

sclist_t* sclist_new() {
    sclist_t* sclist;
    if (NULL==(sclist=malloc(sizeof(struct sclist_s)))) {
        perror("sclist_new OOM");
        abort();
    }
    sclist->first=NULL;
    sclist->last=NULL;

    return sclist;
}

void      sclist_del(sclist_t* sclist) {
    assert(sclist);

    while (sclist->first!=NULL) {
        sclistrecord_t* first = sclist->first;
        sclist->first = sclist->first->next;
        free(first);
    }
    free(sclist);
}

sclistrecord_t* sclist_addrecord(sclist_t* sclist, void* value) {
    sclistrecord_t* record;

    assert(sclist);

    /* Create the record record */
    if (NULL==(record=malloc(sizeof(struct sclistrecord_s)))) {
        perror("sclist_addrecord OOM");
        abort();
    }
    record->value = value;
    record->next = NULL;

    /* Add to the list */
    if (NULL==sclist->first) {
        /* First record to be added in an empty list */
        /* A sentinel record would avoid this test and optimize performances
         * at the cost of sentinel size RAM consumption */
        sclist->first = record;
        sclist->last = record;
    } else {
        /* Add to the end of a non-empty list */
        sclist->last->next = record;
        sclist->last = record;
    }
    return record;
}

sclist_t* sclist_remrecord(sclist_t* sclist, sclistrecord_t* record) {
    sclistrecord_t* cur;
    sclistrecord_t* prev;

    assert(sclist);
    assert(record);

    for (cur=sclist->first, prev=NULL; (cur)&&(cur!=record); prev=cur,cur=cur->next);

    /* If found */
    if (NULL!=cur) {
        /* Remove from the chain */
        if (sclist->first==cur)
            sclist->first = cur->next;
        else
            prev->next = cur->next;
        /* Update the last pointer if needed */
        if (sclist->last==cur)
            sclist->last = prev;
        free(cur);
        return sclist;
    } else {
        fprintf(stderr,"ERROR: sclist_remrecord record not found.\n");
        return NULL;
    }
}

sclistrecord_t* sclist_firstrecord(const sclist_t* sclist) {
    assert(sclist);

    return sclist->first;
}

sclistrecord_t* sclist_nextrecord(const sclistrecord_t* record) {
    assert(record);

    return record->next;
}

void*    sclist_getvalue(sclistrecord_t* record) {
    assert(record);

    return record->value;
}
/* vim: set tw=80: */
