/**
 *    @file  sclist.h
 *   @brief  Basic single chained generic list
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  15/11/2024
 *      Revision:  none
 * Last modified:  2024-11-25 08:56
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifndef __SCLIST_H__
#define __SCLIST_H__

/** Opaque list type
 */
typedef struct sclist_s sclist_t;

/** Opaque list record type
 */
typedef struct sclistrecord_s sclistrecord_t;

/**
 *  @brief  Allocate and initialize the internal list structure
 *
 *  @return Initialized list
 *
 *  Fails and abort the program execution in case of out of memory.
 */
sclist_t* sclist_new();

/**
 *  @brief  Free all the list structure but not the values
 *
 *  @param  sclist List structure (NULL not supported)
 *
 *  Free all the list structures, but DO NOT free the referenced values.
 */
void      sclist_del(sclist_t* sclist);


/**
 *  @brief  Add a value at the end of the list
 *
 *  @param  sclist The list structure to add the value to (NULL not supported)
 *  @param  value  A pointer on the value to add.
 *
 * @return A pointer on the list record
 *
 *  Fails and abort the program execution in case of out of memory.
 */
sclistrecord_t* sclist_addrecord(sclist_t* sclist, void* value);

/**
 *  @brief  Remove a record in a list
 *
 *  @param  sclist The list to remove the value from (NULL not supported)
 *  @param  record A pointer on the record to remove (NULL not supported)
 *
 *  @return A pointer to the sclist
 *      @retval NULL if the record was not found
 *      @retval !NULL if the record was successfully deleted from the sclist
 *
 */
sclist_t*      sclist_remrecord(sclist_t* sclist, sclistrecord_t* record);

/**
 *  @brief  Returns the pointer on the first list record
 *
 *  @param  sclist List structure (NULL not supported)
 *
 *  @return The pointer on the first record in the list
 *      @retval NULL The list is empty
 *      @retval !NULL The pointer on the value record
 */
sclistrecord_t* sclist_firstrecord(const sclist_t* sclist);


/**
 *  @brief  Returns the pointer on the record following the specified one
 *
 *  @param  record Pointer on the current record (NULL not supported)
 *
 *  @return The pointer on the next record if it exists
 *      @retval NULL No next value found (end of list)
 *      @retval !NULL The pointer on the value record
 *
 */
sclistrecord_t* sclist_nextrecord(const sclistrecord_t* record);

/**
 *  @brief  Returns the value pointer stored in the record
 *
 *  @param  record Pointer on the record to get the value from (NULL not supported)
 *
 *  @return Pointer on the value
 */
void*    sclist_getvalue(sclistrecord_t* record);

#endif /* __SCLIST_H__ */
/* vim: set tw=80: */
