/**
 *    @file  suite_libhttp.c
 *   @brief  Check testsuite for libhttp functions
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  20/03/2022
 *      Revision:  none
 * Last modified:  2024-12-03 13:53
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <check.h>
#include <stdlib.h>
#include "checktools.inc"

#include "libhttp.h"
#include <stdio.h>                              /* perror */

/** Dummy test to trigger fixtures
 *
 * This test is defined and added to all testcases in order to execute at
 * least one test test in each testcase of the testsuite, trigger the
 * fixtures and have a decent coverage report.
 */
START_TEST(libhttp_test) {
    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    abort();
}
END_TEST

START_TEST(libhttp_new) {
    HTTP_t* http;
    char* body;
    HTTPHeader_t* header;

    http = HTTP_new();
    ck_assert(http);

    body = HTTP_getbody(http);
    ck_assert(body);
    ck_assert(strcmp(body,"")==0);

    header = HTTP_firstheader(http);
    ck_assert(header==NULL);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_del_null) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    HTTP_del(NULL);
}
END_TEST

START_TEST(libhttp_del_empty) {
    HTTP_t* http;

    http = HTTP_new();
    ck_assert(http);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_del_populated) {
    HTTP_t* http;
    HTTPHeader_t* header;

    http = HTTP_new();
    ck_assert(http);

    header = HTTP_addheader(http,"name1","value1");
    ck_assert(header);

    header = HTTP_addheader(http,"name2","value2");
    ck_assert(header);

    http = HTTP_setbody(http,"<HTML></HTML>");
    ck_assert(http);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_setbody_nullhttp) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    HTTP_setbody(NULL,"");
}
END_TEST

START_TEST(libhttp_setbody_nullbody) {
    HTTP_t* http;

    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    http = HTTP_new();
    ck_assert(http);

    HTTP_setbody(http,NULL);
}
END_TEST

START_TEST(libhttp_setbody_empty) {
    HTTP_t* http;
    char* body;

    http = HTTP_new();
    ck_assert(http);

    http = HTTP_setbody(http,"");
    ck_assert(http);
    body = HTTP_getbody(http);
    ck_assert(strcmp(body,"")==0);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_setbody_valid) {
    HTTP_t* http;
    char* body;

    http = HTTP_new();
    ck_assert(http);

    http = HTTP_setbody(http,"<HTML></HTML>");
    ck_assert(http);
    body = HTTP_getbody(http);
    ck_assert(strcmp(body,"<HTML></HTML>")==0);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_getbody_null) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    HTTP_getbody(NULL);
}
END_TEST

START_TEST(libhttp_getbody_populated) {
    HTTP_t* http;
    char* body;

    http = HTTP_new();
    ck_assert(http);

    body = HTTP_getbody(http);
    ck_assert(body);
    ck_assert(strcmp(body,"")==0);

    http = HTTP_setbody(http,"<HTML></HTML>");
    ck_assert(http);

    body = HTTP_getbody(http);
    ck_assert(body);
    ck_assert(strcmp(body,"<HTML></HTML>")==0);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_addheader_nullhttp) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;
    HTTP_addheader(NULL,"name","value");
}
END_TEST

START_TEST(libhttp_addheader_nullname) {
    HTTP_t* http;

    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    http=HTTP_new();
    HTTP_addheader(http,NULL,"value");
}
END_TEST

START_TEST(libhttp_addheader_nullvalue) {
    HTTP_t* http;

    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    http=HTTP_new();
    HTTP_addheader(http,"name",NULL);
}
END_TEST

START_TEST(libhttp_addheader_emptyname) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"","value");
    ck_assert(NULL==header);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_addheader_emptyvalue) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"name","");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_addheader_valid) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"name","value");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_addbasicauth_nullhttp) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;
    HTTP_addbasicauth(NULL,"login","pass");
}
END_TEST

START_TEST(libhttp_addbasicauth_nulllogin) {
    HTTP_t* http;

    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    http=HTTP_new();
    HTTP_addbasicauth(http,NULL,"pass");
}
END_TEST

START_TEST(libhttp_addbasicauth_nullpass) {
    HTTP_t* http;

    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    http=HTTP_new();
    HTTP_addbasicauth(http,"login",NULL);
}
END_TEST

START_TEST(libhttp_addbasicauth_emptylogin) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addbasicauth(http,"","pass");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("Authorization",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("Basic OnBhc3M=",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_addbasicauth_emptypass) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addbasicauth(http,"login","");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("Authorization",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("Basic bG9naW46",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_addbasicauth_valid) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addbasicauth(http,"login","pass");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("Authorization",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("Basic bG9naW46cGFzcw==",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_setname_nullheader) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;
    HTTPHeader_setname(NULL,"name");
}
END_TEST

START_TEST(libhttp_setname_nullname) {
    HTTP_t* http;
    HTTPHeader_t *header;

    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    http=HTTP_new();
    header = HTTP_addheader(http,"name","value");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));

    header = HTTPHeader_setname(header,NULL);
}
END_TEST

START_TEST(libhttp_setname_emptyname) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"name","value");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));

    header = HTTPHeader_setname(header,"");
    ck_assert(NULL==header);
    header = HTTP_firstheader(http);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_setname_valid) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"name","value");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));

    header = HTTPHeader_setname(header,"newname");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("newname",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_getname_nullheader) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;
    HTTPHeader_getname(NULL);
}
END_TEST

START_TEST(libhttp_getname_valid) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"name","value");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_setvalue_nullheader) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;
    HTTPHeader_setvalue(NULL,"value");
}
END_TEST

START_TEST(libhttp_setvalue_nullvalue) {
    HTTP_t* http;
    HTTPHeader_t *header;

    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    http=HTTP_new();
    header = HTTP_addheader(http,"name","value");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));

    header = HTTPHeader_setvalue(header,NULL);
}
END_TEST

START_TEST(libhttp_setvalue_emptyvalue) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"name","value");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));

    header = HTTPHeader_setvalue(header,"");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_setvalue_valid) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"name","value");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));

    header = HTTPHeader_setvalue(header,"newvalue");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("newvalue",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_getvalue_nullheader) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;
    HTTPHeader_getvalue(NULL);
}
END_TEST

START_TEST(libhttp_getvalue_valid) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"name","value");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_firstheader_nullhttp) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;
    HTTP_firstheader(NULL);
}
END_TEST

START_TEST(libhttp_firstheader_empty) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_firstheader(http);
    ck_assert(NULL==header);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_firstheader_single) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"name","value");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));

    header = HTTP_firstheader(http);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_firstheader_multi) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"name1","value1");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name1",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value1",HTTPHeader_getvalue(header)));
    header = HTTP_addheader(http,"name2","value2");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name2",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value2",HTTPHeader_getvalue(header)));

    header = HTTP_firstheader(http);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name1",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value1",HTTPHeader_getvalue(header)));
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_nextheader_nullheader) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;
    HTTP_nextheader(NULL);
}
END_TEST

START_TEST(libhttp_nextheader_empty) {
    HTTP_t* http;
    HTTPHeader_t *header;

    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    http=HTTP_new();
    header = HTTP_firstheader(http);
    header = HTTP_nextheader(header);
}
END_TEST

START_TEST(libhttp_nextheader_single) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"name","value");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));

    header = HTTP_firstheader(http);
    header = HTTP_nextheader(header);
    ck_assert(NULL==header);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_nextheader_multi) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    header = HTTP_addheader(http,"name1","value1");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name1",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value1",HTTPHeader_getvalue(header)));
    header = HTTP_addheader(http,"name2","value2");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name2",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value2",HTTPHeader_getvalue(header)));
    header = HTTP_addheader(http,"name3","value3");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name3",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value3",HTTPHeader_getvalue(header)));

    header = HTTP_firstheader(http);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name1",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value1",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name2",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value2",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name3",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value3",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL==header);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_findheader_nullheader) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;
    HTTP_findheader(NULL,"");
}
END_TEST

START_TEST(libhttp_findheader_nullname) {
    HTTP_t* http;
    HTTPHeader_t *header;

    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_addheader(http,"name3","value3");
    header = HTTP_firstheader(http);
    HTTP_findheader(header,NULL);
}
END_TEST

START_TEST(libhttp_findheader_emptyname) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_addheader(http,"name3","value3");
    header = HTTP_firstheader(http);
    header = HTTP_findheader(header,"");
    /* Search accepted despite empty header name forbidden, will return NULL
     * because there will always be a miss */
    ck_assert(NULL==header);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_findheader_missingname) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_addheader(http,"name3","value3");
    header = HTTP_firstheader(http);
    header = HTTP_findheader(header,"name4");
    ck_assert(NULL==header);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_findheader_firstname) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_addheader(http,"name3","value3");
    header = HTTP_firstheader(http);
    header = HTTP_findheader(header,"name1");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name1",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value1",HTTPHeader_getvalue(header)));

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_findheader_middlename) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_addheader(http,"name3","value3");
    header = HTTP_firstheader(http);
    header = HTTP_findheader(header,"name2");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name2",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value2",HTTPHeader_getvalue(header)));

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_findheader_lastname) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_addheader(http,"name3","value3");
    header = HTTP_firstheader(http);
    header = HTTP_findheader(header,"name3");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name3",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value3",HTTPHeader_getvalue(header)));

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_findheader_secondmiddle) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_addheader(http,"name1","value4");
    HTTP_addheader(http,"name3","value3");
    header = HTTP_firstheader(http);
    header = HTTP_findheader(header,"name1");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name1",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value1",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    header = HTTP_findheader(header,"name1");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name1",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value4",HTTPHeader_getvalue(header)));

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_findheader_secondlast) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_addheader(http,"name1","value3");
    header = HTTP_firstheader(http);
    header = HTTP_findheader(header,"name1");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name1",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value1",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    header = HTTP_findheader(header,"name1");
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name1",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value3",HTTPHeader_getvalue(header)));

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_remheader_nullhttp) {
    HTTP_t* http;
    HTTPHeader_t *header;

    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    http=HTTP_new();
    header = HTTP_addheader(http,"name1","value1");
    HTTP_remheader(NULL,header);
}
END_TEST

START_TEST(libhttp_remheader_nullheader) {
    HTTP_t* http;

    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    http=HTTP_new();
    HTTP_remheader(http,NULL);
}
END_TEST

START_TEST(libhttp_remheader_missingheader) {
    HTTP_t* http;
    HTTP_t* http2;
    HTTPHeader_t *header;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_addheader(http,"name3","value3");

    http2=HTTP_new();
    HTTP_addheader(http2,"name4","value4");
    header = HTTP_firstheader(http2);

    /* Try to remove http2 header from http (not here) */
    ck_assert(NULL==HTTP_remheader(http,header));

    header = HTTP_firstheader(http);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name1",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value1",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name2",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value2",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name3",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value3",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL==header);
    header = HTTP_firstheader(http2);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name4",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value4",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL==header);

    HTTP_del(http);
    HTTP_del(http2);
}
END_TEST

START_TEST(libhttp_remheader_firstheader) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_addheader(http,"name3","value3");

    header = HTTP_firstheader(http);
    ck_assert(http==HTTP_remheader(http,header));

    header = HTTP_firstheader(http);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name2",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value2",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name3",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value3",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL==header);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_remheader_middleheader) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_addheader(http,"name3","value3");

    header = HTTP_findheader(HTTP_firstheader(http),"name2");
    ck_assert(http==HTTP_remheader(http,header));

    header = HTTP_firstheader(http);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name1",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value1",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name3",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value3",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL==header);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_remheader_lastheader) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_addheader(http,"name3","value3");

    header = HTTP_findheader(HTTP_firstheader(http),"name3");
    ck_assert(http==HTTP_remheader(http,header));

    header = HTTP_firstheader(http);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name1",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value1",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL!=header);
    ck_assert(0==strcmp("name2",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value2",HTTPHeader_getvalue(header)));
    header = HTTP_nextheader(header);
    ck_assert(NULL==header);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_remheader_onlyheader) {
    HTTP_t* http;
    HTTPHeader_t *header;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");

    header = HTTP_firstheader(http);
    ck_assert(http==HTTP_remheader(http,header));

    header = HTTP_firstheader(http);
    ck_assert(NULL==header);

    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_buildheaders_null) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    HTTP_buildheaders(NULL);
}
END_TEST

START_TEST(libhttp_buildheaders_empty) {
    HTTP_t* http;
    char* headers;

    http=HTTP_new();
    headers = HTTP_buildheaders(http);
    ck_assert(0==strcmp(headers,""));
    free(headers);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_buildheaders_single) {
    HTTP_t* http;
    HTTPHeader_t* header;
    char* headers;

    http=HTTP_new();
    header = HTTP_addheader(http,"name1","value1");
    ck_assert(header);
    headers = HTTP_buildheaders(http);
    ck_assert(0==strcmp(headers,"name1: value1\r\n"));
    free(headers);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_buildheaders_multi) {
    HTTP_t* http;
    HTTPHeader_t* header;
    char* headers;

    http=HTTP_new();
    header = HTTP_addheader(http,"name1","value1");
    ck_assert(header);
    header = HTTP_addheader(http,"name2","value2");
    headers = HTTP_buildheaders(http);
    ck_assert(0==strcmp(headers,"name1: value1\r\nname2: value2\r\n"));
    free(headers);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_buildrequest_nullhttp) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    HTTP_buildrequest(NULL, HTTPMETHOD_GET, "/v1", HTTPVERSION_HTTP11);

}
END_TEST

START_TEST(libhttp_buildrequest_nulluri) {
    HTTP_t* http;

    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    http=HTTP_new();
    HTTP_buildrequest(http, HTTPMETHOD_GET, NULL, HTTPVERSION_HTTP11);
}
END_TEST

START_TEST(libhttp_buildrequest_badmethod) {
    HTTP_t* http;
    char* request;

    http=HTTP_new();
    request=HTTP_buildrequest(http, HTTPMETHOD_INVALID+1, "/v1", HTTPVERSION_HTTP11);
    ck_assert(NULL==request);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_buildrequest_badversion) {
    HTTP_t* http;
    char* request;

    http=HTTP_new();
    request=HTTP_buildrequest(http, HTTPMETHOD_GET, "/v1", HTTPVERSION_INVALID+1);
    ck_assert(NULL==request);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_buildrequest_emptyuri) {
    HTTP_t* http;
    char* request;

    http=HTTP_new();
    request=HTTP_buildrequest(http, HTTPMETHOD_GET, "", HTTPVERSION_HTTP11);
    ck_assert(request);
    ck_assert(0==strcmp(request,"GET HTTP/1.1\r\n\r\n"));
    free(request);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_buildrequest_noheader) {
    HTTP_t* http;
    char* request;

    http=HTTP_new();
    request=HTTP_buildrequest(http, HTTPMETHOD_GET, "/v1", HTTPVERSION_HTTP11);
    ck_assert(request);
    ck_assert(0==strcmp(request,"GET /v1 HTTP/1.1\r\n\r\n"));
    free(request);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_buildrequest_singleheader) {
    HTTP_t* http;
    char* request;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    request=HTTP_buildrequest(http, HTTPMETHOD_GET, "/v1", HTTPVERSION_HTTP11);
    ck_assert(request);
    ck_assert(0==strcmp(request,"GET /v1 HTTP/1.1\r\nname1: value1\r\n\r\n"));
    free(request);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_buildrequest_multiheader) {
    HTTP_t* http;
    char* request;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    request=HTTP_buildrequest(http, HTTPMETHOD_GET, "/v1", HTTPVERSION_HTTP11);
    ck_assert(request);
    ck_assert(0==strcmp(request,"GET /v1 HTTP/1.1\r\nname1: value1\r\nname2: value2\r\n\r\n"));
    free(request);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_buildreply_nullhttp) {
    /* SIG6 in debug builds, SIG11 in release builds */
    forktest_only;

    HTTP_buildreply(NULL, HTTPVERSION_HTTP11, SUCCESS_OK);
}
END_TEST

START_TEST(libhttp_buildreply_emptyhttp) {
    HTTP_t* http;
    char* reply;

    http=HTTP_new();
    reply=HTTP_buildreply(http, HTTPVERSION_HTTP11, SUCCESS_OK);
    ck_assert(reply);
    ck_assert(0==strcmp(reply,"HTTP/1.1 200 Dummy Reason\r\n\r\n"));
    free(reply);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_buildreply_invalidversion) {
    HTTP_t* http;
    char* reply;

    http=HTTP_new();
    reply=HTTP_buildreply(http, HTTPVERSION_INVALID+1, SUCCESS_OK);
    ck_assert(NULL==reply);
    HTTP_del(http);
}
END_TEST

START_TEST(libhttp_buildreply_singleheader) {
    HTTP_t* http;
    char* reply;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    reply=HTTP_buildreply(http, HTTPVERSION_HTTP11, SUCCESS_OK);
    ck_assert(reply);
    ck_assert(0==strcmp(reply,"HTTP/1.1 200 Dummy Reason\r\nname1: value1\r\n\r\n"));
    free(reply);
    HTTP_del(http);
}
END_TEST


START_TEST(libhttp_buildreply_multiheader) {
    HTTP_t* http;
    char* reply;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    reply=HTTP_buildreply(http, HTTPVERSION_HTTP11, SUCCESS_OK);
    ck_assert(reply);
    ck_assert(0==strcmp(reply,"HTTP/1.1 200 Dummy Reason\r\nname1: value1\r\nname2: value2\r\n\r\n"));
    free(reply);
    HTTP_del(http);
}
END_TEST


START_TEST(libhttp_buildreply_withbody) {
    HTTP_t* http;
    char* reply;

    http=HTTP_new();
    HTTP_addheader(http,"name1","value1");
    HTTP_addheader(http,"name2","value2");
    HTTP_setbody(http,"<html></html>");
    reply=HTTP_buildreply(http, HTTPVERSION_HTTP11, SUCCESS_OK);
    ck_assert(reply);
    ck_assert(0==strcmp(reply,"HTTP/1.1 200 Dummy Reason\r\nname1: value1\r\nname2: value2\r\n\r\n<html></html>"));
    free(reply);
    HTTP_del(http);
}
END_TEST

/************************************************************************/
/* Begining of test (potentially in child) */
void libhttp_checked_common_setup() {
    signals_catch();
    forktest_gprofdir();
}

/* End of test (potentially in child) */
void libhttp_checked_common_teardown() {
    signals_release();
}
/************************************************************************/
void libhttp_unchecked_common_setup() {
    forktest_init();
}
void libhttp_unchecked_common_teardown() {
}
/************************************************************************/
Suite* libhttp_suite() {
    Suite *s;
    TCase *tc;

    s = suite_create("LibHTTP");

    tc = tcase_create("LibHTTPUninitialized");
    tcase_set_tags(tc,"LibHTTPTag LibHTTPTag2");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, libhttp_checked_common_setup, libhttp_checked_common_teardown);
    tcase_add_unchecked_fixture(tc, libhttp_unchecked_common_setup, libhttp_unchecked_common_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test_raise_signal(tc, libhttp_test,6);
    tcase_add_test(tc, libhttp_new);
    tcase_add_test_raise_signal(tc, libhttp_del_null,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_del_empty);
    tcase_add_test(tc, libhttp_del_populated);
    tcase_add_test_raise_signal(tc, libhttp_setbody_nullhttp,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_setbody_nullbody,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_setbody_empty);
    tcase_add_test(tc, libhttp_setbody_valid);
    tcase_add_test_raise_signal(tc, libhttp_getbody_null,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_getbody_populated);
    tcase_add_test_raise_signal(tc, libhttp_addheader_nullhttp,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_addheader_nullname,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_addheader_nullvalue,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_addheader_emptyname);
    tcase_add_test(tc, libhttp_addheader_emptyvalue);
    tcase_add_test(tc, libhttp_addheader_valid);
    tcase_add_test_raise_signal(tc, libhttp_addbasicauth_nullhttp,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_addbasicauth_nulllogin,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_addbasicauth_nullpass,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_addbasicauth_emptylogin);
    tcase_add_test(tc, libhttp_addbasicauth_emptypass);
    tcase_add_test(tc, libhttp_addbasicauth_valid);
    tcase_add_test_raise_signal(tc, libhttp_setname_nullheader,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_setname_nullname,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_setname_emptyname);
    tcase_add_test(tc, libhttp_setname_valid);
    tcase_add_test_raise_signal(tc, libhttp_getname_nullheader,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_getname_valid);
    tcase_add_test_raise_signal(tc, libhttp_setvalue_nullheader,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_setvalue_nullvalue,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_setvalue_emptyvalue);
    tcase_add_test(tc, libhttp_setvalue_valid);
    tcase_add_test_raise_signal(tc, libhttp_getvalue_nullheader,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_getvalue_valid);
    tcase_add_test_raise_signal(tc, libhttp_firstheader_nullhttp,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_firstheader_empty);
    tcase_add_test(tc, libhttp_firstheader_single);
    tcase_add_test(tc, libhttp_firstheader_multi);
    tcase_add_test_raise_signal(tc, libhttp_nextheader_nullheader,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_nextheader_empty,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_nextheader_single);
    tcase_add_test(tc, libhttp_nextheader_multi);
    tcase_add_test_raise_signal(tc, libhttp_findheader_nullheader,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_findheader_nullname,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_findheader_emptyname);
    tcase_add_test(tc, libhttp_findheader_missingname);
    tcase_add_test(tc, libhttp_findheader_firstname);
    tcase_add_test(tc, libhttp_findheader_middlename);
    tcase_add_test(tc, libhttp_findheader_lastname);
    tcase_add_test(tc, libhttp_findheader_secondmiddle);
    tcase_add_test(tc, libhttp_findheader_secondlast);
    tcase_add_test_raise_signal(tc, libhttp_remheader_nullhttp,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_remheader_nullheader,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_remheader_missingheader);
    tcase_add_test(tc, libhttp_remheader_firstheader);
    tcase_add_test(tc, libhttp_remheader_middleheader);
    tcase_add_test(tc, libhttp_remheader_lastheader);
    tcase_add_test(tc, libhttp_remheader_onlyheader);
    tcase_add_test_raise_signal(tc, libhttp_buildheaders_null,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_buildheaders_empty);
    tcase_add_test(tc, libhttp_buildheaders_single);
    tcase_add_test(tc, libhttp_buildheaders_multi);
#if 0
    tcase_add_test_raise_signal(tc, libhttp_parseheaders_null,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_parseheaders_empty);
    tcase_add_test(tc, libhttp_parseheaders_single);
    tcase_add_test(tc, libhttp_parseheaders_multi);
    tcase_add_test(tc, libhttp_parseheaders_split);
    tcase_add_test(tc, libhttp_parseheaders_invalid_nocolon);
    tcase_add_test(tc, libhttp_parseheaders_invalid_novalue);
    tcase_add_test(tc, libhttp_parseheaders_invalid_noname);
#endif
    tcase_add_test_raise_signal(tc, libhttp_buildrequest_nullhttp,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_buildrequest_nulluri,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_buildrequest_badmethod);
    tcase_add_test(tc, libhttp_buildrequest_badversion);
    tcase_add_test(tc, libhttp_buildrequest_emptyuri);
    tcase_add_test(tc, libhttp_buildrequest_noheader);
    tcase_add_test(tc, libhttp_buildrequest_singleheader);
    tcase_add_test(tc, libhttp_buildrequest_multiheader);
#if 0
    tcase_add_test_raise_signal(tc, libhttp_parserequest_nullrequest,SIG_ABRTSEGV);
#endif
    tcase_add_test_raise_signal(tc, libhttp_buildreply_nullhttp,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_buildreply_emptyhttp);
    tcase_add_test(tc, libhttp_buildreply_invalidversion);
    tcase_add_test(tc, libhttp_buildreply_singleheader);
    tcase_add_test(tc, libhttp_buildreply_multiheader);
    tcase_add_test(tc, libhttp_buildreply_withbody);
#if 0
    tcase_add_test_raise_signal(tc, libhttp_parsereply_nullreply,SIG_ABRTSEGV);
#endif

    return s;
}
/* vim: set tw=80: */
